from typing_extensions import Self

class InfoRequest:
    def __init__(self) -> None: ...
    @classmethod
    def FromString(cls, s: str) -> Self: ...
    @classmethod
    def SerializeToString(cls, s: Self) -> str: ...
    ...

class JobRequest:
    working_dir: str
    files_name: str
    index: int
    def __init__(self, working_dir: str, files_name: str, index: int) -> None: ...
    @classmethod
    def FromString(cls, s: str) -> Self: ...
    @classmethod
    def SerializeToString(cls, s: Self) -> str: ...
    ...

class JobResponse:
    status: str
    message: str
    def __init__(self, status: str, message: str) -> None: ...
    @classmethod
    def FromString(cls, s: str) -> Self: ...
    @classmethod
    def SerializeToString(cls, s: Self) -> str: ...
    ...

class Contact:
    name: str
    email: str
    def __init__(self, name: str, email: str) -> None: ...
    @classmethod
    def FromString(cls, s: str) -> Self: ...
    @classmethod
    def SerializeToString(cls, s: Self) -> str: ...
    ...

class License:
    name: str
    url: str
    def __init__(self, name: str, url: str) -> None: ...
    @classmethod
    def FromString(cls, s: str) -> Self: ...
    @classmethod
    def SerializeToString(cls, s: Self) -> str: ...
    ...

class InfoResponse:
    title: str
    description: str
    version: str
    contact: Contact
    license: License
    def __init__(
        self,
        title: str,
        description: str,
        version: str,
        contact: Contact,
        license: License,
    ) -> None: ...
    @classmethod
    def FromString(cls, s: str) -> Self: ...
    @classmethod
    def SerializeToString(cls, s: Self) -> str: ...
    ...
